library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity sha256digester is
   port 
   (
      clk       : in  std_logic;
      rx_w      : in  std_logic_vector(511 downto 0);
      rx_state  : in  std_logic_vector(255 downto 0);
      tx_w      : out std_logic_vector(511 downto 0);
      tx_state  : out std_logic_vector(255 downto 0);
      k         : in  std_logic_vector(31 downto 0)
   );
end sha256digester;

architecture arch of sha256digester is
   
   signal e0_w  : std_logic_vector(31 downto 0);
   signal e1_w  : std_logic_vector(31 downto 0);
   signal ch_w  : std_logic_vector(31 downto 0);
   signal maj_w : std_logic_vector(31 downto 0);
   signal s0_w  : std_logic_vector(31 downto 0);
   signal s1_w  : std_logic_vector(31 downto 0);
   
   signal t1    : std_logic_vector(31 downto 0);
   signal t2    : std_logic_vector(31 downto 0);
   signal new_w : std_logic_vector(31 downto 0);
   
begin

   isum0 : entity work.sum0
      port map (rx_state(31 downto 0), e0_w);
   

   isum1 : entity work.sum1
      port map (rx_state(159 downto 128), e1_w);
   
   
   ich : entity work.ch
      port map (rx_state(159 downto 128), rx_state(191 downto 160), rx_state(223 downto 192), ch_w);
   
   
   imaj : entity work.maj
      port map (rx_state(31 downto 0), rx_state(63 downto 32), rx_state(95 downto 64), maj_w);
   
   
   isigma0 : entity work.sigma0
      port map (rx_w(63 downto 32), s0_w);
   
   
   isigma1 : entity work.sigma1
      port map (rx_w(479 downto 448), s1_w);
   
  
   t1 <= std_logic_vector(unsigned(rx_state(255 downto 224)) + unsigned(e1_w) + unsigned(ch_w) + unsigned(rx_w(31 downto 0)) + unsigned(K));
  
   t2 <= std_logic_vector(unsigned(e0_w) + unsigned(maj_w));
   
   new_w <= std_logic_vector(unsigned(s1_w) + unsigned(rx_w(319 downto 288)) + unsigned(s0_w) + unsigned(rx_w(31 downto 0)));
  
   process (clk)
   begin
      if rising_edge(clk) then
         
         tx_w(511 downto 480) <= new_w;
         tx_w(479 downto 0)   <= rx_w(511 downto 32);
         
         tx_state(255 downto 224) <= rx_state(223 downto 192);
         tx_state(223 downto 192) <= rx_state(191 downto 160);
         tx_state(191 downto 160) <= rx_state(159 downto 128);
         tx_state(159 downto 128) <= std_logic_vector(unsigned(rx_state(127 downto 96)) + unsigned(t1));
         tx_state(127 downto 96)  <= rx_state(95 downto 64);  
         tx_state(95 downto 64)   <= rx_state(63 downto 32);  
         tx_state(63 downto 32)   <= rx_state(31 downto 0);   
         tx_state(31 downto 0)    <= std_logic_vector(unsigned(t1) + unsigned(t2));
      
      
      end if;
   end process;
  
end architecture;



