library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity miner_top is
   port 
   (
      osc_clk          : in  std_logic;
      midstate_vw_in   : in  std_logic_vector(255 downto 0);
      data2_vw_in      : in  std_logic_vector(255 downto 0);
      golden_nonce_out : out std_logic_vector(31 downto 0)
   );
end miner_top;

architecture arch of miner_top is
  
   signal state : std_logic_vector(255 downto 0) := (others => '0');
   signal data  : std_logic_vector(511 downto 0) := (others => '0');
   signal nonce : std_logic_vector( 31 downto 0) := (others => '0');
  
   signal hash_clk : std_logic;
   signal clk      : std_logic;
  
   signal hash  : std_logic_vector(255 downto 0);
   signal hash2 : std_logic_vector(255 downto 0);
  
   signal is_golden_ticket : std_logic := '0';
  
   signal midstate_buf  : std_logic_vector(255 downto 0) := (others => '0');
   signal data_buf      : std_logic_vector(255 downto 0) := (others => '0');
   
   signal midstate_vw   : std_logic_vector(255 downto 0);
   signal data2_vw      : std_logic_vector(255 downto 0);
   
   signal golden_nonce : std_logic_vector( 31 downto 0) := (others => '0');
   
   signal rx_input_sha2 : std_logic_vector(511 downto 0);
  
begin
   
   hash_clk <= osc_clk;
   clk      <= hash_clk;

   isha256transform : entity work.sha256transform
      port map (clk, state, data, hash);
      
   rx_input_sha2 <=  x"0000010000000000000000000000000000000000000000000000000080000000" & hash;
      
   isha256transform2 : entity work.sha256transform
      port map (clk, x"5be0cd191f83d9ab9b05688c510e527fa54ff53a3c6ef372bb67ae856a09e667", rx_input_sha2, hash2);
   
  
   midstate_vw <= midstate_vw_in;
   data2_vw <= data2_vw_in;
   
   golden_nonce_out <= golden_nonce;
   
   process (clk)
   begin
      if rising_edge(clk) then
      
         midstate_buf <= midstate_vw;
         data_buf <= data2_vw;
         
         state <= midstate_buf;
         data <= x"000002800000000000000000000000000000000000000000000000000000000000000000000000000000000080000000" & nonce & data_buf(95 downto 0);
         nonce <= std_logic_vector(unsigned(nonce) + 1);
         
         if (hash2(255 downto 224) = (255 downto 224 => '0')) then
            is_golden_ticket <= '1';
         else
            is_golden_ticket <= '0';
         end if;
         
         if(is_golden_ticket = '1') then
            golden_nonce <= nonce;
         end if;
         
      end if;
   end process;
  
end architecture;
